--[[
RealisticSeederUtil

Utility for my Realistic Seeder

Author:		Ifko[nator]
Datum:		04.07.2020
Version:	v1.0

History:	v1.0 @ 04.07.2020 - initial implementation in FS 19
]]

RealisticSeederUtil = {};
RealisticSeederUtil.currentModName = g_currentModName;
RealisticSeederUtil.languageShort = g_languageShort;

RealisticSeederUtil.validStoreCategories = {
    "seeders",
    "planters",
	"potatoHarvesting",
	"weeders"
};

getfenv(0)["RealisticSeederUtil"] = RealisticSeederUtil;

function RealisticSeederUtil.getDebugPriority(xmlFile, path)
	return Utils.getNoNil(getXMLInt(xmlFile, path), 0);
end;

function RealisticSeederUtil.printError(errorMessage, isWarning, isInfo, luaName)
	local prefix = "::ERROR:: ";
	
	if isWarning then
		prefix = "::WARNING:: ";
	elseif isInfo then
		prefix = "::INFO:: ";
	end;
	
	print(prefix .. "from the " .. luaName .. ".lua: " .. tostring(errorMessage));
end;

function RealisticSeederUtil.printDebug(debugMessage, priority, addString, luaName)
	if priority >= 1 then
		local prefix = "";
		
		if addString then
			prefix = "::DEBUG:: from the " .. luaName .. ".lua: ";
		end;
		
		print(prefix .. tostring(debugMessage));
	end;
end;

function RealisticSeederUtil.getSpecByName(self, specName, currentModName)
    local spec = self["spec_" .. Utils.getNoNil(currentModName, RealisticSeederUtil.currentModName) .. "." .. specName];

	if spec ~= nil then
        return spec;
    end;

    return self["spec_" .. specName];
end;

function RealisticSeederUtil.getIsValidStoreCategory(storeCategory)
    for _, validStoreCategory in pairs(RealisticSeederUtil.validStoreCategories) do
        if storeCategory == validStoreCategory then
            return true;
        end;
    end;

    return false;
end;

function RealisticSeederUtil.foundFunctionAdditionalTank(xmlFile)
    local foundFunctionAdditionalTank = false;

	local additionalTankNumber = 0;

	while true do
		local functionAditionalTankKey = "vehicle.storeData.functions.function(" .. additionalTankNumber .. ")";
		
		if not hasXMLProperty(xmlFile, functionAditionalTankKey) then
			break;
		end;

		if getXMLString(xmlFile, functionAditionalTankKey) == "$l10n_function_additionalTank" then
			foundFunctionAdditionalTank = true;

			break;
		end;

		additionalTankNumber = additionalTankNumber + 1;
    end;
    
    return foundFunctionAdditionalTank;
end;

function RealisticSeederUtil.getSeedFillUnitIndex(xmlFile, object, specFillUnit)
	local seedFillUnitIndex = 1;
	
	local fillUnitConfigurationId = Utils.getNoNil(object.configurations["fillUnit"], 1);
	local baseKey = string.format("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(%d).fillUnits", fillUnitConfigurationId -1);

	local fillUnitNumber = 0;

	while true do
		local fillUnitKey = string.format("%s.fillUnit(%d)", baseKey, fillUnitNumber);
		
        if not hasXMLProperty(xmlFile, fillUnitKey) then
            break;
		end;

		for fillTypeIndex, _ in pairs(specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes) do
			if g_fillTypeManager:getFillTypeByIndex(fillTypeIndex).name == "SEEDS" then
				seedFillUnitIndex = fillUnitNumber + 1;

				break;
			end;
		end;

		fillUnitNumber = fillUnitNumber + 1;
    end;
    
    return seedFillUnitIndex;
end;

function RealisticSeederUtil.changeMaterial(object, specLanguageChecker)
    local oldMaterial = getMaterial(specLanguageChecker.materialNode, 0);
	local newMaterial = getMaterial(specLanguageChecker.englishNode, 0);

	for _, component in pairs(object.components) do
		ConfigurationUtil.replaceMaterialRec(object, component.node, oldMaterial, newMaterial);
	end;
end;