--[[
RegisterFillTypes.lua
	
Author: 	Ifko[nator]
Date:		14.05.2020
Version:	1.0

History:	v1.0 @ 14.05.2020 - initial implementation in FS 19
]]

RegisterFillTypes = {};
RegisterFillTypes.currentModDirectory = g_currentModDirectory;
RegisterFillTypes.debugPriority = 0;

function RegisterFillTypes:loadedMap(mission, node)
	if RealisticSeederUtil == nil then
        return;
	end;
	
    local modDesc = loadXMLFile("modDesc", RegisterFillTypes.currentModDirectory .. "modDesc.xml");

    RegisterFillTypes.debugPriority = Utils.getNoNil(getXMLInt(modDesc, "modDesc.fillTypes#debugPriority"), RegisterFillTypes.debugPriority);

    local fillTypeNumber = 0;

    while true do
        local fillTypeKey = "modDesc.fillTypes.fillType(" .. tostring(fillTypeNumber) .. ")";

        if not hasXMLProperty(modDesc, fillTypeKey) then
            break;
        end;

        local fillType = Utils.getNoNil(string.upper(getXMLString(modDesc, fillTypeKey .. "#name")), "");

        if fillType ~= "" then
            if g_fillTypeManager:getFillTypeByName(fillType) == nil then
                local title = Utils.getNoNil(getXMLString(modDesc, fillTypeKey .. "#title"), "");

				if title ~= "" then
					if g_i18n:hasText(title) then
						title = g_i18n:getText(title);
					elseif string.sub(title, 1, 6) == "$l10n_" then
                        title = g_i18n:getText(string.sub(title, 7));
                    end;

                    local showOnPriceTable = Utils.getNoNil(getXMLBool(modDesc, fillTypeKey .. "#showOnPriceTable"), true);
                    local pricePerLiter = Utils.getNoNil(getXMLFloat(modDesc, fillTypeKey .. "#pricePerLiter"), 0.6);
                    local hudOverlayFilename = Utils.getNoNil(getXMLString(modDesc, fillTypeKey .. ".image#hud"), "$dataS2/menu/hud/fillTypes/hud_fill_seeds.png");
                    local hudOverlayFilenameSmall = Utils.getNoNil(getXMLString(modDesc, fillTypeKey .. ".image#hudSmall"), "$dataS2/menu/hud/fillTypes/hud_fill_seeds_sml.png");
                    local massPerLiter = Utils.getNoNil(getXMLFloat(modDesc, fillTypeKey .. ".physics#massPerLiter"), 0.1) / 1000;
                    local maxPhysicalSurfaceAngle = Utils.getNoNil(getXMLFloat(modDesc, fillTypeKey .. ".physics#maxPhysicalSurfaceAngle"), 35);
                    local palletFilename = getXMLString(modDesc, fillTypeKey .. ".pallet#filename");

                    g_fillTypeManager:addFillType(
                        fillType, 
                        title, 
                        showOnPriceTable, 
                        pricePerLiter, 
                        massPerLiter, 
                        maxPhysicalSurfaceAngle, 
                        hudOverlayFilename, 
                        hudOverlayFilenameSmall, 
                        RegisterFillTypes.currentModDirectory, 
                        nil, 
                        {1, 1, 1}, 
                        palletFilename, 
                        false
                    );

                    RealisticSeederUtil.printDebug("Register fillType '" .. fillType .. "'(" .. title .. ") successfully!", RegisterFillTypes.debugPriority, true, "RegisterFillTypes");
                else
                    RealisticSeederUtil.printError("Missing 'title' attribute for fill type '" .. fillType .. "' (number '" .. fillTypeNumber + 1 .. "')! Skipping this fill type now!", false, false, "RegisterFillTypes");
                end;
            else
                RegisterFillTypes.printError("The fill type '" .. fillType .. "' (number '" .. fillTypeNumber + 1 .. "') is already registered! Skipping this fill type now!", false, false, "RegisterFillTypes");
            end;
        else
            RegisterFillTypes.printError("Missing 'name' attribute for fill type number '" .. fillTypeNumber + 1 .. "'! Skipping this fill type now!", false, false, "RegisterFillTypes");
        end;

        fillTypeNumber = fillTypeNumber + 1;
    end;

    delete(modDesc);
end;

function RegisterFillTypes:addFillType(superFunc, name, title, showOnPriceTable, pricePerLiter, massPerLiter, maxPhysicalSurfaceAngle, hudOverlayFilename, hudOverlayFilenameSmall, baseDirectory, customEnv, fillPlaneColors, palletFilename, isBaseType)
	if name == nil or name == "" or name:find("[^%w_.]") then
		RegisterFillTypes.printError("'" .. tostring(name) .. "' is not a valid name for a fillType. Ignoring fillType!", true, false, "RegisterFillTypes");

		return nil;
	end;

	name = name:upper()

	if isBaseType and self.nameToFillType[name] ~= nil then
		RegisterFillTypes.printError("FillType '" .. tostring(name) .. "' already exists. Ignoring fillType!", true, false, "RegisterFillTypes");

		return nil;
	end;

	local fillType = self.nameToFillType[name];

	if fillType == nil then
		--## no limit for us :D
		--if #self.fillTypes > 256 then
		--	RegisterFillTypes.printError("FillTypeManager.addFillType too many fill types. Only 256 fill types are supported", false, false, "RegisterFillTypes");

		--	return;
		--end;

		fillType = {
			name = name,
			index = #self.fillTypes + 1,
			title = g_i18n:convertText(title, customEnv)
		};

		self.nameToFillType[name] = fillType;
		self.nameToIndex[name] = fillType.index;
		self.indexToName[fillType.index] = name;

		table.insert(self.fillTypes, fillType);
	end;

	fillType.showOnPriceTable = Utils.getNoNil(showOnPriceTable, Utils.getNoNil(fillType.showOnPriceTable, false));
	fillType.pricePerLiter = Utils.getNoNil(pricePerLiter, Utils.getNoNil(fillType.pricePerLiter, 0));
	fillType.massPerLiter = Utils.getNoNil(massPerLiter, Utils.getNoNil(fillType.massPerLiter, 0.0001)) * FillTypeManager.MASS_SCALE;
	fillType.maxPhysicalSurfaceAngle = Utils.getNoNilRad(maxPhysicalSurfaceAngle, Utils.getNoNil(fillType.maxPhysicalSurfaceAngle, math.rad(30)));
	fillType.hudOverlayFilename = Utils.getFilename(hudOverlayFilename, baseDirectory) or fillType.hudOverlayFilename;
	fillType.hudOverlayFilenameSmall = Utils.getFilename(hudOverlayFilenameSmall, baseDirectory) or fillType.hudOverlayFilenameSmall;

	if fillType.index ~= FillType.UNKNOWN then
		if fillType.hudOverlayFilename == nil or fillType.hudOverlayFilename == "" then
			RegisterFillTypes.printError("FillType '" .. name .. "' has no valid image assigned!", true, false, "RegisterFillTypes");
		end;

		if fillType.hudOverlayFilenameSmall == nil or fillType.hudOverlayFilenameSmall == "" then
			RegisterFillTypes.printError("FillType '" .. name .. "' has no valid small image assigned!", true, false, "RegisterFillTypes");
		end;
	end;

	if palletFilename ~= nil then
		fillType.palletFilename = Utils.getFilename(palletFilename, baseDirectory) or fillType.palletFilename;
	end;

	fillType.previousHourPrice = fillType.pricePerLiter;
	fillType.startPricePerLiter = fillType.pricePerLiter;
	fillType.totalAmount = FillTypeManager.FILLTYPE_START_TOTAL_AMOUNT;
	fillType.fillPlaneColors = {};

	if fillPlaneColors ~= nil then
		fillType.fillPlaneColors[1] = fillPlaneColors[1] or fillType.fillPlaneColors[1];
		fillType.fillPlaneColors[2] = fillPlaneColors[2] or fillType.fillPlaneColors[2];
		fillType.fillPlaneColors[3] = fillPlaneColors[3] or fillType.fillPlaneColors[3];
	else
		fillType.fillPlaneColors[1] = fillType.fillPlaneColors[1] or 1;
		fillType.fillPlaneColors[2] = fillType.fillPlaneColors[2] or 1;
		fillType.fillPlaneColors[3] = fillType.fillPlaneColors[3] or 1;
	end;

	return fillType;
end;

FillTypeManager.addFillType = Utils.overwrittenFunction(FillTypeManager.addFillType, RegisterFillTypes.addFillType);

FSBaseMission.loadMapFinished = Utils.prependedFunction(FSBaseMission.loadMapFinished, RegisterFillTypes.loadedMap);

addModEventListener(RegisterFillTypes);