--[[
StoreManagerExtension.lua
	
Author: 	Ifko[nator]
Date:		04.07.2020
Version:	1.0

History:	v1.0 @ 04.07.2020 - initial implementation in FS 19
]]

StoreManagerExtension = {};
StoreManagerExtension.currentModDirectory = g_currentModDirectory;

function StoreManagerExtension:onStartMission()
    local modDesc = loadXMLFile("modDesc", StoreManagerExtension.currentModDirectory .. "modDesc.xml");

    StoreManagerExtension.debugPriority = RealisticSeederUtil.getDebugPriority(modDesc, "modDesc.realisticSeeder#debugPriority");

    if RealisticSeederUtil.languageShort ~= "de" then
        local newStoreImages = {};
        local storeImageNumber = 0;

        while true do
            local storeImageKey = "modDesc.storeItems.storeItem(" .. tostring(storeImageNumber) .. ")";

            if not hasXMLProperty(modDesc, storeImageKey) then
                break;
            end;

            local germanStoreImage = Utils.getNoNil(getXMLString(modDesc, storeImageKey .. "#germanStoreImage"), "");
            local englishStoreImage = Utils.getNoNil(getXMLString(modDesc, storeImageKey .. "#englishStoreImage"), "");

            local storeImage = {};

            if germanStoreImage ~= "" and englishStoreImage ~= "" then
                storeImage.germanStoreImage = germanStoreImage;
                storeImage.englishStoreImage = englishStoreImage;

                table.insert(newStoreImages, storeImage);
            end;

            storeImageNumber = storeImageNumber + 1;
        end;

        for _, modStoreItem in pairs(g_storeManager.items) do
            if string.find(modStoreItem.imageFilename, LanguageChecker.currentModName) then
                for _, storeImage in pairs(newStoreImages) do
                    if string.find(modStoreItem.imageFilename, storeImage.germanStoreImage) then 
                        --## swap store image
                        modStoreItem.imageFilename = Utils.getFilename(storeImage.englishStoreImage, StoreManagerExtension.currentModDirectory);

                        RealisticSeederUtil.printDebug("Swaped store image  " .. storeImage.germanStoreImage .. " with " .. modStoreItem.imageFilename.. ".", StoreManagerExtension.debugPriority, true, "StoreManagerExtension");
                    end;
                end;
            end;
        end;
    end;

    local fillTypeNames = getXMLString(modDesc, "modDesc.realisticSeeder#fillTypes");
    local fillTypes = g_fillTypeManager:getFillTypesByNames(fillTypeNames, "Warning: invalid fillType '%s'.");
    
    if fillTypes ~= nil then
        for _, fillType in pairs(fillTypes) do
            --## check if selected fruit is available in current map
	    	local canFruitBePlanted = false;

        	if g_currentMission.fruits ~= nil then
                local seedsFruitType = g_fruitTypeManager:getFruitTypeByName(string.sub(g_fillTypeManager:getFillTypeByIndex(fillType).name, 6));

				if seedsFruitType ~= nil and g_currentMission.fruits[seedsFruitType.index] ~= nil then
        	        if g_currentMission.fruits[seedsFruitType.index].id ~= nil and g_currentMission.fruits[seedsFruitType.index].id ~= 0 then
                        canFruitBePlanted = true;
                    end;
                end;
	    	end;

            if not canFruitBePlanted and g_fillTypeManager:getFillTypeByIndex(fillType).palletFilename ~= nil then
                local storeItem = g_storeManager:getItemByXMLFilename(g_fillTypeManager:getFillTypeByIndex(fillType).palletFilename);

                --## remove pallet since its fill type will be not supportet
                g_storeManager:removeItemByIndex(storeItem.id);

                RealisticSeederUtil.printDebug("Removed store item " .. g_fillTypeManager:getFillTypeByIndex(fillType).palletFilename .. " from the shop due to not supportet fillType (" .. g_fillTypeManager:getFillTypeByIndex(fillType).title .. ")!", StoreManagerExtension.debugPriority, true, "StoreManagerExtension");
            end;
        end;
    end;

    if g_fillTypeManager:getFillTypeByName("SEEDS").palletFilename ~= nil then
        if g_fillTypeManager:getFillTypeByName("SEEDS").palletFilename == "data/objects/bigBagContainer/bigBagContainerSeeds.xml" then
           
            --## change default seed big bag with pioneer pallet for unloading
            g_fillTypeManager:getFillTypeByName("SEEDS").palletFilename = "data/objects/pallets/pioneer/pioneerPallet.xml";
        end;
    end;

    delete(modDesc);
end;

FSBaseMission.onStartMission = Utils.prependedFunction(FSBaseMission.onStartMission, StoreManagerExtension.onStartMission);

addModEventListener(StoreManagerExtension);